/* Copyright (C) Peter Rank Software - All Rights Reserved
 * Written by Peter Rank <peter@softmanufaktur.de>, 2016
 */
import React from 'react';
import LCalFormatter from '../calendar/lcalformatter.js';
import LCalHelper from '../calendar/lcalhelper.js';
import Typography from '@material-ui/core/Typography';
import Paper from '@material-ui/core/Paper';
import LCal from "../calendar/lcal";
import LCalInterval from "../calendar/lcalinterval";

class DateTimeBlock extends React.Component {
  render() {
    if (this.props.lcal) {
      const now = new LCal().setJulianMinutes(LCalHelper.getNowMinutes()).setTimeZone("Europe/Berlin");
      const endForIntervalComputation = this.props.displayNowToLatestEnd ? this.props.lcal.getLatestLCal().setPrecision(this.props.lcal.precision) : this.props.lcal.getEarliestLCal().setPrecision(this.props.lcal.precision); //Die Genauigkeit des mouseLCal ändern, je nach dem wie groß der Abstand ist (damit nicht immer Minuten mit angezeigt werden, wenn z.B. der Abstand 1000 Jahre beträgt)

      const adaptedEndForIntervalComputation = LCalHelper.adaptPrecisionByDistance(endForIntervalComputation, now);
      const interval = now.after(this.props.lcal) ? new LCalInterval(adaptedEndForIntervalComputation, now) : new LCalInterval(now, adaptedEndForIntervalComputation);
      const isPast = this.props.lcal && this.props.lcal.before(now);
      const duration = LCalFormatter.formatDuration(interval);
      return /*#__PURE__*/React.createElement(Paper, {
        style: {
          display: "flex",
          flexDirection: "column",
          flex: "0 0 50%"
        }
      }, /*#__PURE__*/React.createElement("div", {
        style: {
          margin: 1,
          padding: 7,
          flex: "1 1 auto"
        }
      }, LCalFormatter.formatType(this.props.lcal) && /*#__PURE__*/React.createElement(Typography, {
        style: {
          color: "gray"
        }
      }, LCalFormatter.formatType(this.props.lcal)), /*#__PURE__*/React.createElement("div", {
        style: {
          display: "flex",
          justifyContent: "center"
        }
      }, this.props.lcal.precision > 10 &&
      /*#__PURE__*/
      //Mehr als monatsgenau (=tagesgenau und genauer) -> Datum mit anzeigen
      React.createElement("div", {
        style: {
          marginRight: 8
        }
      }, /*#__PURE__*/React.createElement(Typography, {
        style: {
          fontSize: 30
        }
      }, LCalFormatter.formatDay(this.props.lcal), ".")), /*#__PURE__*/React.createElement("div", {
        style: {
          display: "flex",
          flexDirection: "column"
        }
      }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement(Typography, null, (this.props.lcal.precision > 9 ? LCalFormatter.formatMonthNameL(this.props.lcal) + " " : "") + LCalFormatter.formatYear(this.props.lcal) //Mehr als jahresgenau (=monatsgenau und genauer) -> Wochentag mit anzeigen
      )), /*#__PURE__*/React.createElement("div", null, this.props.lcal.precision > 10 &&
      /*#__PURE__*/
      //Mehr als monatsgenau (=tagesgenau und genauer) -> Wochentag mit anzeigen
      React.createElement(Typography, {
        style: {
          color: "gray"
        }
      }, LCalFormatter.formatDayNameL(this.props.lcal))))), this.props.lcal.precision > 11 &&
      /*#__PURE__*/
      //Mehr als tagesgenau (=stundengenau und besser) -> Uhrzeit mit anzeigen
      React.createElement("div", null, /*#__PURE__*/React.createElement("hr", {
        style: {
          margin: "4px 0",
          padding: 0
        }
      }), /*#__PURE__*/React.createElement(Typography, null, LCalFormatter.formatTime(this.props.lcal) + " Uhr")), /*#__PURE__*/React.createElement("hr", {
        style: {
          margin: "4px 0",
          padding: 0
        }
      }), /*#__PURE__*/React.createElement("div", {
        style: {
          display: "flex",
          justifyContent: "center",
          color: "gray"
        }
      }, /*#__PURE__*/React.createElement(Typography, {
        style: {
          fontSize: 12
        }
      }, duration === "0" || duration === "~ 0" ? "jetzt" : (isPast ? "vor " : "in ") + duration))));
    } else {
      return /*#__PURE__*/React.createElement(Paper, {
        style: {
          display: "flex",
          flex: "0 0 50%",
          justifyContent: "center"
        }
      }, /*#__PURE__*/React.createElement("div", {
        style: {
          padding: 7,
          margin: 1,
          flex: "1 1 auto",
          justifyContent: "center"
        }
      }));
    }
  }

}

export default DateTimeBlock;